import processing.core.*; 
import processing.xml.*; 

import ddf.minim.analysis.*; 
import ddf.minim.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class LiveSpectrogram_logf extends PApplet {

/**
 * LiveSpectrogram_logf
 * 
 * Listens to sound in.
 * Draws a real-time scrolling spectrogram with log freq axis as it is playing.
 *
 * Dan Ellis dpwe@ee.columbia.edu 2010-03-29
 */
 


 
Minim minim;
AudioInput sound;
FFT fft;

// Configuration: spectrogram size (pixels)
int colmax = 900;
int rowmax = 192;
int labelwidth = 80;

int lastrow;

// buffer size (= FFT size, must be power of 2)
//int bufferSize = 1024;
int bufferSize = 2048;

int minBW = 80;
int binsPerOct = 24;
int pixelsPerBin = 2;

// Variables
int[][] sgram = new int[rowmax][colmax];
int col = 0;
int leftedge = 1;

int lastmillis;
boolean paused = false;

int CTSIZE = 64;
int [][] colortable = {
{10, 0, 0},
{21, 0, 0},
{31, 0, 0},
{42, 0, 0},
{53, 0, 0},
{63, 0, 0},
{74, 0, 0},
{85, 0, 0},
{95, 0, 0},
{106, 0, 0},
{116, 0, 0},
{127, 0, 0},
{138, 0, 0},
{148, 0, 0},
{159, 0, 0},
{170, 0, 0},
{180, 0, 0},
{191, 0, 0},
{201, 0, 0},
{212, 0, 0},
{223, 0, 0},
{233, 0, 0},
{244, 0, 0},
{255, 0, 0},
{255, 10, 0},
{255, 21, 0},
{255, 32, 0},
{255, 42, 0},
{255, 53, 0},
{255, 64, 0},
{255, 74, 0},
{255, 85, 0},
{255, 96, 0},
{255, 106, 0},
{255, 117, 0},
{255, 128, 0},
{255, 138, 0},
{255, 149, 0},
{255, 160, 0},
{255, 170, 0},
{255, 181, 0},
{255, 192, 0},
{255, 202, 0},
{255, 213, 0},
{255, 224, 0},
{255, 234, 0},
{255, 245, 0},
{255, 256, 0},
{255, 256, 16},
{255, 256, 32},
{255, 256, 48},
{255, 256, 64},
{255, 256, 80},
{255, 256, 96},
{255, 256, 112},
{255, 256, 128},
{255, 256, 144},
{255, 256, 160},
{255, 256, 176},
{255, 256, 192},
{255, 256, 208},
{255, 256, 224},
{255, 256, 240},
{255, 256, 256}
};

public void setup()
{
  size(colmax + labelwidth, rowmax*pixelsPerBin, P2D);
//  size(980, 192, P2D);
  textMode(SCREEN);
//  textFont(createFont("SanSerif", 12));
 
  minim = new Minim(this);
  
  //sound = minim.loadFile("sound.mp3",bufferSize);
  // make it repeat
  //sound.loop();

  sound = minim.getLineIn(Minim.MONO, bufferSize);

  fft = new FFT(sound.bufferSize(), sound.sampleRate());
  // use a tapered window (to avoid "splatter")
  fft.window(FFT.HAMMING);
  
  // set log scaling range
  fft.logAverages(minBW, binsPerOct);
  
  println("Num bins = "+fft.avgSize());
  
  lastrow = min(rowmax, fft.avgSize());
  
  lastmillis = millis();
  
  // Try to sync the frame rate to the file
  frameRate(sound.sampleRate()/sound.bufferSize());
  println("Target frame period = "+Math.round(1000*sound.bufferSize()/sound.sampleRate())+" ms");

}

// Common routine used to draw the spectrogram in two parts
// (before and after the "leftedge" wrap point)
public void drawPart(int[][] sgram, int firstcol, int ncols, int xbase)
{
  for (int i = 0; i < ncols; i++) {
    // draw the column of spectrogram cells
    for (int j = 0; j < lastrow; j++) {
      int val = sgram[j][i+firstcol];
      //stroke(min(255,val));
      int ctix = min(CTSIZE-1,(CTSIZE*val)/256);   
      stroke(colortable[ctix][0],colortable[ctix][1],colortable[ctix][2]);
      line(xbase + i, height - j*pixelsPerBin, xbase + i, height - (j+1)*pixelsPerBin);
    }
  }
}

public int myFreqToIndex(double freq) 
{
  // which log avg bin contains this frequency?
  return (int)(binsPerOct*Math.log(freq/minBW*1.8f)/Math.log(2));
}

public void draw()
{
  if (!paused) {
  
    background(0);
    stroke(255);
    // perform a forward FFT on the samples in the input buffer  
    fft.forward(sound.mix);
  
    for(int i = 0; i < lastrow; i++)
    {
      // fill in the new column of spectral values
      double specval = fft.getAvg(i);
      sgram[i][col] = (int)Math.round(Math.max(0,4*20*Math.log10(30*fft.getAvg(i))));
    }
    
    // next time will be the next column
    col = col + 1; 
    // wrap back to the first column when we get to the end
    if (col == colmax) { col = 0; }
    
    // Draw points.  
    // leftedge is the column in the ring-filled array that is drawn at the extreme left
    // start from there, and draw to the end of the array
    drawPart(sgram, leftedge, colmax-leftedge, 0);
    drawPart(sgram, 0, leftedge, colmax-leftedge);
  
    // Next time around, we move the left edge over by one, to have the whole thing
    // scroll left
    leftedge = leftedge + 1; 
    // Make sure it wraps around
    if (leftedge == colmax) { leftedge = 0; }
    
    // Add frequency axis
    int x = colmax + 2; // to right of spectrogram display
    stroke(255);
    line(x,0,x,height); // vertical line
    // x,y address of text is immediately to the left of the middle of the letters 
    textAlign(LEFT,CENTER);
    for (double freq = 100.0f; freq < sound.sampleRate()/2; freq *= 2.0f) {
      int y = height - pixelsPerBin*myFreqToIndex(freq); // which bin holds this frequency
      line(x,y,x+3,y); // tick mark
      text(Math.round(freq)+" Hz", x+5, y); // add text label
    }
    // update framerate
    if (false) {
      int nowmillis = millis();
      text((nowmillis-lastmillis)+" ms", 10, 10);
      lastmillis = nowmillis;
    }
  }
}

public void keyPressed()
{
  // spacebar toggles play/pause
  if (key == ' ') {
    paused = !paused;
    if (paused) {
      //sound.pause();
    } else {
      //sound.play();
    }
  }
}
 
 
public void stop()
{
  // always close Minim audio classes when you finish with them
  sound.close();
  minim.stop();
 
  super.stop();
}


  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "LiveSpectrogram_logf" });
  }
}
